#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Auxilliary functions for handling Dicom data.
"""
import os
import dicom


def compare_dicom_key(dcm):
    """ Specifying the sorting key for CT images.
    """
    return float(dcm.ImagePositionPatient[2])


def read_dicom_folder(path):
    """ Reads a folder with dicom files.
    Identifies each dicom file with .dcm suffix and returns a dict containing a dicom object.
    Dicom object may be "CT", "RTSTRUCT", "RTDOSE" or "RTPLAN".
    Corresponding keys for lookup are "images", "rtss", "rtdose" or "rtplan" repsectively.
    CT objects are lists of images. They will be sorted by the position in patient given
    by the ImagePositionPatient[2] tag.

    :returns: A dict containing dicom objects and corresponding keys 'images','rtss','rtdose' or 'rtplan'.
    """
    if not os.path.isdir(path):
        raise IOError("Folder does not exist")

    # list of allowed dicom file extentions names
    # all in lower case
    dicom_suffix = ('.dcm', '.ima', '.v2')

    data = {}
    folder = os.listdir(path)
    for item in folder:
        if os.path.splitext(item)[1].lower() in dicom_suffix:
            dcm = dicom.read_file(os.path.join(path, item), force=True)
            # TODO figureout what was it about (see below)
            # if dicom.__version__ >= "0.9.5":
            # dcm = dicom.read_file(os.path.join(path, item), force=True)
            # else:
            #     dcm = dicom.read_file(os.path.join(path, item))
            if dcm.Modality == "CT":
                if "images" not in data:
                    data["images"] = []
                data["images"].append(dcm)
            elif dcm.Modality == "RTSTRUCT":
                data["rtss"] = dcm
            elif dcm.Modality == "RTDOSE":
                data["rtdose"] = dcm
            elif dcm.Modality == "RTPLAN":
                data["rtplan"] = dcm
    if "images" in data:
        data["images"].sort(key=compare_dicom_key)
    return data



#!/usr/bin/env python
#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Script for converting Voxelplan / TRiP98 Ctx and Vdx data to a DICOM file.
"""
import os
import sys
import logging
import argparse

import pytrip as pt

logger = logging.getLogger(__name__)


def main(args=sys.argv[1:]):
    """ Main function for trip2dicom.py
    """
    # parse arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("ctx_data", help="location of CT file (header or data) in TRiP98 format", type=str)
    parser.add_argument("outputdir", help="write resulting DICOM files to this directory", type=str)
    parser.add_argument("-v", "--verbosity", action='count', help="increase output verbosity", default=0)
    parser.add_argument('-V', '--version', action='version', version=pt.__version__)
    parsed_args = parser.parse_args(args)

    if parsed_args.verbosity == 1:
        logging.basicConfig(level=logging.INFO)
    elif parsed_args.verbosity > 1:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig()

    output_folder = parsed_args.outputdir

    _, data_file_name = pt.CtxCube.parse_path(parsed_args.ctx_data)
    data_file_path = pt.CtxCube.discover_file(data_file_name)

    if not os.path.exists(data_file_path):
        logger.error("CTX file missing")
        return 1

    logger.info("Convert CT images...")
    c = pt.CtxCube()
    c.read(data_file_name)

    if not os.path.exists(output_folder):
        os.makedirs(output_folder)
    c.write_dicom(output_folder)

    ctx_basename = os.path.splitext(data_file_path)[0]
    ctx_path = ctx_basename + ".vdx"
    if os.path.exists(ctx_path):
        logger.info("Convert VDX structures...")
        v = pt.VdxCube(cube=c)
        v.read(ctx_path)
        v.write_dicom(output_folder)
    else:
        logger.info("No VDX data found for conversion.")

    logger.info("Done")
    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
TODO: documentation here.
"""


class DoseCube(object):
    def __init__(self, dosecube, type):
        self.dosecube = dosecube
        self.type = type

    def get_dosecube(self):
        return self.dosecube

    def get_type(self):
        return self.type

    def set_type(self, type):
        self.type = type

    def set_dose(self, value):
        self.dosecube.target_dose = float(value)

    def get_dose(self):
        return self.dosecube.target_dose

    def calculate_dvh(self, voi):
        dvh, min_dose, max_dose, mean, area = self.dosecube.calculate_dvh(voi)
        return dvh, min_dose, max_dose, mean, area



#! /usr/bin/env python
#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
This script plots the raster scan file for verication of the spot delivery of the ion accelerator.
"""
import sys
import argparse
import logging

import pytrip as pt


def main(args=sys.argv[1:]):
    # there are some cases when this script is run on systems without DISPLAY variable being set
    # in such case matplotlib backend has to be explicitly specified
    # we do it here and not in the top of the file, as inteleaving imports with code lines is discouraged
    import matplotlib
    matplotlib.use('Agg')
    from pylab import plt, ylabel, grid, xlabel, array

    parser = argparse.ArgumentParser()
    parser.add_argument("rst_file", help="location of rst file in TRiP98 format", type=str)
    parser.add_argument("output_file", help="location of PNG file to save", type=str)
    parser.add_argument("-s", "--submachine", help="Select submachine to plot.", type=int, default=1)
    parser.add_argument("-f", "--factor", help="Factor for scaling the blobs. Default is 1000.", type=int, default=1000)
    parser.add_argument("-v", "--verbosity", action='count', help="increase output verbosity", default=0)
    parser.add_argument('-V', '--version', action='version', version=pt.__version__)
    args = parser.parse_args(args)

    file = args.rst_file
    sm = args.submachine
    fac = args.factor

    a = pt.Rst()
    a.read(file)

    # convert data in submachine to a nice array
    b = a.machines[sm]

    x = []
    y = []
    z = []
    for _x, _y, _z in b.raster_points:
        x.append(_x)
        y.append(_y)
        z.append(_z)

    title = "Submachine: {:d} / {:d} - Energy: {:.3f} MeV/u".format(sm, len(a.machines), b.energy)
    print(title)
    cc = array(z)

    cc = cc / cc.max() * fac

    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.scatter(x, y, c=cc, s=cc, alpha=0.75)
    ylabel("mm")
    xlabel("mm")
    grid(True)
    plt.title(title)
    plt.savefig(args.output_file)
    plt.close()


if __name__ == '__main__':
    logging.basicConfig()
    sys.exit(main(sys.argv[1:]))



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
The CTX module contains the CtxCube class which is inherited from the Cube class.
It is used for handling CT-data, both Voxelplan and Dicom.
"""
import os
import datetime
import copy

import numpy as np

from pytrip.error import InputError
from pytrip.cube import Cube


class CtxCube(Cube):
    """ Class for handling CT-data. In TRiP98 these are stored in VOXELPLAN format with the .ctx suffix.
    This class can also handle Dicom files.
    """
    data_file_extension = ".ctx"
    header_file_extension = ".hed"

    def __init__(self, cube=None):
        super(CtxCube, self).__init__(cube)
        self.type = "CTX"

    def read_dicom(self, dcm):
        """ Imports CT-images from Dicom object.

        :param Dicom dcm: a Dicom object
        """
        if "images" not in dcm:
            raise InputError("Data doesn't contain ct data")
        if not self.header_set:
            self.read_dicom_header(dcm)

        self.cube = np.zeros((self.dimz, self.dimy, self.dimx), dtype=np.int16)
        intersect = float(dcm["images"][0].RescaleIntercept)
        slope = float(dcm["images"][0].RescaleSlope)

        for i in range(len(dcm["images"])):
            data = np.array(dcm["images"][i].pixel_array) * slope + intersect
            self.cube[i][:][:] = data
        if self.slice_pos[1] < self.slice_pos[0]:
            self.slice_pos.reverse()
            self.zoffset = self.slice_pos[0]
            self.cube = self.cube[::-1]

    def create_dicom(self):
        """ Creates a Dicom object from self.

        This function can be used to convert a TRiP98 CTX file to Dicom format.

        :returns: A Dicom object.
        """
        data = []

        ds = self.create_dicom_base()
        ds.Modality = 'CT'
        ds.SamplesperPixel = 1
        ds.BitsAllocated = self.num_bytes * 8
        ds.BitsStored = self.num_bytes * 8
        ds.HighBit = self.num_bytes * 8 - 1
        ds.PatientPosition = 'HFS'
        ds.RescaleIntercept = 0.0
        ds.ImageType = ['ORIGINAL', 'PRIMARY', 'AXIAL']
        ds.PatientPosition = 'HFS'
        # ds.SeriesInstanceUID is created in the top-level cube class
        # ds.SeriesInstanceUID = '2.16.840.1.113662.2.12.0.3057.1241703565.43'
        ds.RescaleSlope = 1.0
        ds.PixelRepresentation = 1
        ds.SOPClassUID = '1.2.840.10008.5.1.4.1.1.2'  # CT Image Storage SOP Class

        # .HED files do not carry any time stamp (other than the usual file meta data)
        # so let's just fill it with current times. (Can be overridden by user)
        ds.SeriesDate = datetime.datetime.today().strftime('%Y%m%d')
        ds.ContentDate = datetime.datetime.today().strftime('%Y%m%d')
        ds.SeriesTime = datetime.datetime.today().strftime('%H%M%S')
        ds.ContentTime = datetime.datetime.today().strftime('%H%M%S')

        # Eclipse tags

        # Manufacturer of the equipment that produced the composite instances.
        ds.Manufacturer = self.creation_info  # Manufacturer tag, 0x0008,0x0070 (type LO - Long String)

        ds.KVP = ''  # KVP tag 0x0018, 0x0060

        ds.AcquisitionNumber = '1'  # AcquisitionNumber tag 0x0020, 0x0012 (type IS - Integer String)

        for i in range(len(self.cube)):
            _ds = copy.deepcopy(ds)
            _ds.ImagePositionPatient = ["{:.3f}".format(self.xoffset),
                                        "{:.3f}".format(self.yoffset),
                                        "{:.3f}".format(self.slice_pos[i])]

            _ds.SOPInstanceUID = '2.16.1.113662.2.12.0.3057.1241703565.' + str(i + 1)
            _ds.SliceLocation = str(self.slice_pos[i])
            _ds.InstanceNumber = str(i + 1)
            pixel_array = np.zeros((_ds.Rows, _ds.Columns), dtype=self.pydata_type)
            pixel_array[:][:] = self.cube[i][:][:]
            _ds.PixelData = pixel_array.tostring()
            _ds.pixel_array = pixel_array
            data.append(_ds)
        return data

    def write(self, path):
        """ Write CT-data to disk, in TRiP98/Voxelplan format.

        This method will build and write both the .hed and .ctx file.

        :param str path: path to header file, data file or basename (without extension)
        """

        header_file, ctx_file = self.parse_path(path_name=path)
        self.write_trip_header(header_file)
        self.write_trip_data(ctx_file)

    def write_dicom(self, directory):
        """ Write CT-data to disk, in Dicom format.

        :param str directory: directory to write to. If directory does not exist, it will be created.
        """

        if not os.path.exists(directory):
            os.makedirs(directory)

        dcm_list = self.create_dicom()
        for dcm_item in dcm_list:
            dcm_item.save_as(os.path.join(directory, "CT.PYTRIP.{:d}.dcm".format(dcm_item.InstanceNumber)))



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
TODO: documentation here.
"""
from pytrip.tripexecuter.pytripobj import pytripObj
from pytrip.tripexecuter.fieldcollection import FieldCollection
from pytrip.tripexecuter.voicollection import VoiCollection
from pytrip.tripexecuter.tripexecuter import TripExecuter, InputError
from pytrip.dos import DosCube
from pytrip.let import LETCube
from pytrip.tripexecuter.dosecube import DoseCube
import os


class TripPlan(pytripObj):
    def __init__(self, name="", comment=""):
        self.save_fields = ["fields", "vois", "remote", "servername", "username", "password", "working_dir",
                            "ddd_folder", "spc_folder", "sis_file", "res_tissue_type", "target_tissue_type", "name",
                            "comment"]
        self.save_fields.extend(["iterations", "opt_method", "opt_princip", "dose_alg", "bio_alg", "opt_alg", "eps",
                                 "geps", "out_phys_dose", "out_bio_dose", "out_dose_mean_let", "out_field",
                                 "out_mean_let", "dose_percent"])
        self.fields = FieldCollection(self)
        self.vois = VoiCollection(self)
        self.remote = False
        self.servername = ""
        self.username = ""
        self.password = ""
        self.working_dir = "$HOME/tmp"
        self.ddd_folder = ""
        self.spc_folder = ""
        self.sis_file = ""
        self.hlut_file = "$TRIP98/DATA/19990211.hlut"
        self.dedx_file = "$TRIP98/DATA/DEDX/20040607.dedx"
        self.res_tissue_type = ""
        self.target_tissue_type = ""
        self.active = False
        self.name = name
        self.comment = comment
        self.dosecube = None
        self.dosecubes = []
        self.optimize = True
        self.letcube = None
        self.iterations = 500
        self.opt_method = "Phys"
        self.opt_princip = "H2OBased"
        self.dose_alg = "classic"
        self.bio_alg = "classic"
        self.opt_alg = "classic"
        self.eps = 1e-3
        self.geps = 1e-4

        self.out_phys_dose = True
        self.out_bio_dose = False
        self.out_dose_mean_let = False
        self.out_field = False
        self.out_mean_let = False
        self.window = []
        self.dose_percent = {}
        self.target_dose_cube = None

    def Init(self, parent):
        self.parent = parent
        self.vois.Init()
        self.fields.Init()

    def set_optimize(self, optimize):
        self.optimize = optimize

    def get_optimize(self):
        return self.optimize

    def set_window(self, window):
        self.window = window

    def get_window(self):
        return self.window

    def set_target_dose_cube(self, dos):
        self.target_dose_cube = dos

    def get_target_dose_cube(self):
        return self.target_dose_cube

    def set_dose_percent(self, ion, dose_percent):
        if dose_percent == "":
            del self.dose_percent[ion]
        self.dose_percent[ion] = float(dose_percent)

    def get_dose_percent(self, ion):
        if ion in self.dose_percent:
            return self.dose_percent[ion]
        return None

    def get_all_dose_percent(self):
        return self.dose_percent

    def remove_dose_percent(self, ion):
        del self.dose_percent[ion]

    def save_plan(self, images, path):
        self.save_exec(images, path + ".exec")
        self.save_data(images, path + ".exec")

    def save_exec(self, images, path):
        t = TripExecuter(images)
        t.set_plan(self)
        t.save_exec(path)

    def save_data(self, images, path):
        t = TripExecuter(images)
        t.set_plan(self)
        t.save_data(path)
        for dos in self.dosecubes:
            dos.get_dosecube().write(path + "." + dos.get_type() + ".dos")
        if self.letcube is not None:
            self.letcube.write(path + "." + "dosmeanlet.dos")

    def get_projectile(self):
        if len(self.fields):
            return self.fields.get_fields()[0].get_projectile()

    def get_out_phys_dose(self):
        return self.out_phys_dose

    def set_out_phys_dose(self, value):
        self.out_phys_dose = value

    def get_out_bio_dose(self):
        return self.out_bio_dose

    def set_out_bio_dose(self, value):
        self.out_bio_dose = value

    def get_out_dose_mean_let(self):
        return self.out_dose_mean_let

    def set_out_dose_mean_let(self, value):
        self.out_dose_mean_let = value

    def get_out_mean_let(self):
        return self.out_dose_mean_let

    def set_out_mean_let(self, value):
        self.out_dose_mean_let = value

    def get_out_field(self):
        return self.out_field

    def set_out_field(self, value):
        self.out_field = value

    def set_res_tissue_type(self, res_tissue_type):
        self.res_tissue_type = res_tissue_type

    def get_res_tissue_type(self):
        return self.res_tissue_type

    def set_target_tissue_type(self, target_tissue_type):
        self.target_tissue_type = target_tissue_type

    def get_target_tissue_type(self):
        return self.target_tissue_type

    def is_remote(self):
        return self.remote

    def set_remote_state(self, state):
        self.remote = state

    def get_server(self):
        return self.servername

    def set_server(self, server):
        """ Set server name where TRiP98 is installed, if not installed locally."""
        self.servername = server

    def get_username(self):
        """ Get username for login on remote TRiP98 server."""
        return self.username

    def set_username(self, username):
        """ Set username for login on remote TRiP98 server."""
        self.username = username

    def get_password(self):
        """ Get password for login on remote TRiP98 server. Note, this stored unencrypted in cleartext."""
        return self.password

    def set_password(self, password):
        """ Set password for login on remote TRiP98 server. Note, this stored unencrypted in cleartext."""
        self.password = password

    def get_working_dir(self):
        """ Get working directory of TRiP98."""
        return self.working_dir

    def set_working_dir(self, working_dir):
        """ Set working directory of TRiP98."""
        self.working_dir = working_dir

    def get_iterations(self):
        """ Get number of iterations for optimization in TRiP98."""
        return self.iterations

    def get_eps(self):
        return self.eps

    def get_geps(self):
        return self.geps

    def get_bio_algorithm(self):
        return self.bio_alg

    def get_opt_algorithm(self):
        return self.opt_alg

    def get_opt_method(self):
        return self.opt_method

    def set_opt_method(self, method):
        self.opt_method = method

    def get_dose_algorithm(self):
        return self.dose_alg

    def active_dose_change(self, dos):
        self.dosecube = dos

    def load_let(self, path):
        if hasattr(self, "letcube"):
            if self.letcube is not None:
                self.remove_let(self.letcube)
        let = LETCube()
        let.read(os.path.splitext(path)[0] + ".dos")

        self.letcube = let
        return let

    def remove_let(self, let):
        self.letcube = None

    def get_let(self):
        if hasattr(self, "letcube"):
            return self.letcube
        return None

    def get_let_cube(self):
        if self.letcube is not None:
            return self.letcube.cube
        return None

    def get_dose(self, type=""):
        if len(self.dosecubes):
            if type == "":
                return self.dosecube
            for cube in self.dosecubes:
                if cube.get_type() == type:
                    return cube
        return None

    def get_dose_cube(self):
        if self.dosecube is not None:
            return self.dosecube.get_dosecube().cube
        return None

    def remove_dose(self, dos):
        self.dosecubes.remove(dos)
        if self.dosecube is dos:
            self.dosecube = None
        self.clean_cache()

    def remove_dose_by_type(self, type):
        dos = None
        for cube in self.dosecubes:
            if cube.get_type() == type:
                dos = cube
        if dos is not None:
            self.dosecubes.remove(dos)
            self.clean_cache()
        return dos

    def clean_cache(self):
        for voi in self.vois:
            voi.clean_cache()

    def add_dose(self, dos, t=""):
        if type(dos) is DosCube:
            dos = DoseCube(dos, t)
        if hasattr(self, "dosecube"):
            if self.dosecube is not None:
                self.remove_dose_by_type(dos.get_type())
        self.dosecubes.append(dos)

        self.set_active_dose(dos)

    def set_active_dose(self, dos):
        self.dosecube = dos
        self.clean_cache()

    def load_dose(self, path, t, target_dose=0.0):
        dos = DosCube()
        dos.read(os.path.splitext(path)[0] + ".dos")
        d = DoseCube(dos, t)
        d.set_dose(target_dose)
        self.add_dose(d)

    def set_eps(self, value):
        try:
            value = float(value)
            if value < 0:
                raise Exception()
            self.eps = value
        except Exception as e:
            raise InputError("eps shoud be a number Larger than 0" + e)

    def set_geps(self, value):
        try:
            value = float(value)
            if value < 0:
                raise Exception()
            self.geps = value

        except Exception as e:
            raise InputError("geps shoud be a number Larger than 0" + e)

    def set_iterations(self, value):
        try:
            value = int(value)
            if value < 0:
                raise Exception()
            self.iterations = value
        except Exception as e:
            raise InputError("iterations shoud be a number Larger than 0" + e)

    def set_bio_algorithm(self, value):
        self.bio_alg = value

    def set_dose_algorithm(self, value):
        self.dose_alg = value

    def set_opt_algorithm(self, value):
        self.opt_alg = value

    def get_opt_princip(self):
        return self.opt_princip

    def set_opt_princip(self, value):
        self.opt_princip = value

    def set_phys_bio(self, value):
        self.phys_bio = value

    def set_name(self, name):
        if self.parent.get_plan(name) is None:
            self.name = name
            return True
        return False

    def get_name(self):
        return self.name

    def get_vois(self):
        return self.vois

    def get_fields(self):
        return self.fields

    def add_voi(self, voi):
        return self.vois.add_voi(voi)

    def add_field(self, field):
        """ Add a new field to the plan object."""
        return self.fields.add_field(field)

    def remove_field(self, field):
        """ Remove field from the plan object."""
        self.fields.remove(field)

    def get_field_datasource(self):
        return self.fields

    def get_voi_datasource(self):
        return self.vois

    def destroy(self):
        self.vois.destroy()
        self.fields.destroy()

    def set_ddd_folder(self, path):  # TODO: propose to rename "folder" -> "dir"
        """ Set directory containing the depth dose kernels in .ddd format """
        self.ddd_folder = path

    def get_ddd_folder(self):
        """ Get directory containing the depth dose kernels in .ddd format """
        return self.ddd_folder

    def set_dedx_file(self, filename):  # TODO: propose to rename "file" -> "path"
        """ Set path to stopping power tables in .dedx format """
        self.dedx_file = filename

    def get_dedx_file(self, filename):
        """ Get path to stopping power tables in .dedx format """
        return self.dedx_file

    def set_hlut_file(self, filename):
        """ Set path to Hounsfield lookup tables in .hlut format """
        self.hlut_file = filename

    def get_hlut_file(self, filename):
        """ Get path to Hounsfield lookup tables in .hlut format """
        return self.hlut_file

    def get_sis_file(self):
        """ Get path to SIS tables."""
        return self.sis_file

    def set_sis_file(self, path):
        """ Set path to SIS tables."""
        self.sis_file = path

    def set_spc_folder(self, path):
        """ Set directory containing the beam kernel spectrum files in .spc format """
        self.spc_folder = path

    def get_spc_folder(self):
        """ Get directory containing the beam kernel spectrum files in .spc format """
        return self.spc_folder



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
TODO: documentation here.
"""
from pytrip.tripexecuter.pytripobj import pytripObj
from pytrip.tripexecuter.tripvoi import TripVoi
from pytrip.util import get_class_name


class VoiCollection(pytripObj):
    def __init__(self, parent):
        self.save_fields = ["vois"]
        self.parent = parent
        self.vois = []

    def Init(self):
        pass

    def __iter__(self):
        self.iter = 0
        return self

    def __len__(self):
        return len(self.vois)

    def __getitem__(self, i):
        return self.vois[i]

    def __next__(self):  # Python3 iterator method
        if self.iter >= len(self.vois):
            raise StopIteration
        self.iter += 1
        return self.vois[self.iter - 1]

    next = __next__  # Python2 iterator method

    def get_vois(self):
        return self.vois

    def get_voi_by_name(self, name):
        for voi in self:
            if voi.get_name() == name:
                return voi
        return None

    def move_voi(self, voi, step):
        for i, v in enumerate(self.vois):
            if v is voi:
                break
        i2 = i + step
        if i < 0 or i >= len(self.vois):
            return False
        else:
            tmp = self.vois[i2]
            self.vois[i2] = self.vois[i]
            self.vois[i] = tmp
            return True

    def get_target_name(self):
        return ""

    def add_voi(self, voi):
        if voi in self.vois:
            return False
        if get_class_name(self.parent) == "TripPlan":
            if TripVoi not in voi.__class__.__bases__:
                voi = TripVoi(voi)
            for v in self.vois:
                if v._voi is voi._voi:
                    return False
        self.vois.append(voi)
        return True

    def delete_voi(self, voi):
        self.vois.remove(voi)



#! /usr/bin/env python
#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
""" Reads .gd files and can convert them into (xmgrace readable) ascii data.

Can be used in the command line

Mandatory arguments:
Name      Type       Default  Description
filename  STRING     ""       File name of .gd file, should be first.

Optional arguments:
Name      Type       Default  Description
exp       STRING     "exp"    if "exp": Export data of .gd into a .dat file
agr       STRING     ""       if "agr": Export data of .gd into a .agr file
LET       STRING     "LET"    if "LET": Export also data entitled "DoseLET"

Example:
1) reading the file foo.gd and exporting it as ascii file foo.dat
python gd2dat.py foo.gd

2) reading the file foo.gd and exporting it as xmgrace file foo.agr
python gd2dat.py foo.gd agr
"""
import sys
import os
import logging
import argparse

import pytrip as pt


class ReadGd(object):
    """This class reads .gd files."""
    # TODO: this class could go into main pytrip/ as well.

    def __init__(self, gd_filename, exp=False, agr=False, let=False):
        """ setup of object """
        self.filename = gd_filename
        self.head = []
        self.legend = []
        self.indata = []
        self.xdata = []
        self.data = []
        self.title = ""
        self.xlabel = ""
        self.ylabel = ""
        self.h = 0
        self.export_bool = exp
        self.agr = agr
        self.let_bool = let

        print('\n# Running the conversion scrip gd2dat.py')
        if self.filename is None:
            print("No file name has been specified")
        else:
            print("# Reading the file " + self.filename + " with gd2dat.py")
            self.read()

    def read(self):
        """ Reads the filename set when constructing the ReadGd class
        """
        if os.path.isfile(self.filename) is False:
            raise IOError("Could not find file " + self.filename)

        gd_file = open(self.filename, 'r')
        gd_lines = gd_file.readlines()
        gd_file.close()

        line = gd_lines[0]
        line_len = len(line)
        self.title = line[:line_len - 1]

        for line in gd_lines[1:]:
            s = 0
            while s < 10:
                if line[s] == ' ':
                    s += 1
                else:
                    break

            if line[s] == 'x' or line[s] == 'X':
                line_len = len(line)
                self.xlabel = line[s + 2:line_len - 1]
            elif line[s] == 'y' or line[s] == 'Y':
                line_len = len(line)
                self.ylabel = line[s + 2:line_len - 1]
            elif line[s] in ('h', 'H', 'a', 'A'):
                line_len = len(line)
                p = 0
                while p < line_len:
                    l2 = line[p:p + 2]
                    legend_bool = False
                    if l2 == 'x ' or l2 == 'X ':
                        self.head.append('x')
                        self.h += 1
                        self.legend.append("x")
                        legend_bool = False
                    elif l2 == 'y(' or l2 == 'Y(':
                        self.head.append('y')
                        self.h += 1
                        legend_bool = True
                    elif l2 == 'm(' or l2 == 'M(':
                        self.head.append('m')
                        self.h += 1
                        legend_bool = True
                    elif l2 == 'n(' or l2 == 'N(':
                        self.head.append('n')
                        self.h += 1
                        legend_bool = True
                    elif l2 == 'l(' or l2 == 'L(':
                        self.head.append('l')
                        self.h += 1
                        legend_bool = True

                    if legend_bool:
                        legend_bool = False
                        t = p + 1
                        while t < line_len:
                            if line[t] == ')':
                                self.legend.append(line[p + 2:t])
                                if self.let_bool:
                                    if line[p + 2:t] == "mb^DoseLETme^":
                                        self.head[self.h - 1] = "y"
                                        self.legend[self.h - 1] = "DoseLET"
                                    if line[p + 2:t] == "mb^LETme^":
                                        self.head[self.h - 1] = "y"
                                        self.legend[self.h - 1] = "FluenceLET"

                                p = t
                                break
                            else:
                                t += 1
                    p += 1

            elif line[s] == 'n' or line[s] == 'N':
                break
            elif line[s].isdigit() or line[s] == '-':
                line_len = len(line)
                linelist = (line[:line_len - 1]).split(' ')
                self.indata.append(linelist)

        num = 0
        while num < self.h:
            ydat = []
            if self.head[num] == "x":
                for ele in self.indata:
                    self.xdata.append(ele[num])
                    ydat.append(float(ele[num]))
                self.data.append(ydat)
            elif self.head[num] in ('y', 'm', 'n'):  # normal data
                for ele in self.indata:
                    ydat.append(float(ele[num]))
                self.data.append(ydat)
            num += 1

    def export(self, out_file_name=None):
        """ Export data to out_file_name
        :params str out_file_name: full path to output file including file extension
        """
        if out_file_name is None:
            len_file_name = len(self.filename)
            out_file_name = self.filename[:len_file_name - 2]
            if self.agr:
                out_file_name += "agr"
                print('# Writing data in a ".agr" file fragment: ' + out_file_name)
            else:
                out_file_name += "dat"
                print('# Writing data in a ".dat" file: ' + out_file_name)

        out_file = open(out_file_name, 'w')

        if self.let_bool:
            print('# Exporting also LET data  ')

        if self.agr:
            header = "# Grace project file\n# \n@version 50122 \n"
            # header += "@page size 842, 595 \n@page scroll 5% \n"
            header += "@page inout 5% \n@link page off \n"

            str_out = header + '@    title  "' + self.title + ' "\n'

            str_out += '@    xaxis  label "' + self.xlabel
            str_out += ' "\n@    xaxis  label char size 1.500000\n'
            str_out += '@    xaxis  ticklabel char size 1.250000\n'

            str_out += '@    yaxis  label "' + self.ylabel
            str_out += ' "\n@    yaxis  label char size 1.500000\n'
            str_out += '@    yaxis  ticklabel char size 1.250000\n'

            out_file.write(str_out)

        num = -1
        counter = 0

        while num < self.h - 1:

            num += 1
            # The following feature is still under development
            #            title = self.legend[num]
            #            if (title == "Survival" or title == "SpecDose" \
            #                    or title == "PhysDose" ):
            #                print '# Skip data with the title "' + title + '"'
            #                continue
            #            print self.head[num]  # debug line

            if self.head[num] == "x":
                continue

            elif self.head[num] == 'y' or self.head[num] == 'm':  # normal data

                if self.head[num] == 'm':
                    str_hd = self.legend[num]
                    j = 0
                    for sign in str_hd:
                        if sign == '*':
                            break
                        j += 1
                    _legend = str_hd[j + 1:]
                else:
                    _legend = self.legend[num]

                str_out = ' \n'
                if not self.agr:
                    str_out += '# '
                str_out += '@    s' + str(counter) + ' legend  "'
                str_out += _legend + '"\n'
                out_file.write(str_out)

                str_out = '@    s' + str(counter) + ' comment "'
                if not self.agr:
                    str_out = '# ' + str_out
                str_out += self.filename + ' "\n'
                for i, ele in enumerate(self.data[num]):
                    str_out += self.xdata[i] + ' ' + str(ele) + '\n'
                out_file.write(str_out)
                counter += 1

            else:
                continue

            # special handling for 'm'
            #
            # A column with the header 'm' should be multiplied with the
            # column to the left of it (cf GD manual).
            #
            if self.head[num] == 'm':

                str_out = ' \n'
                if not self.agr:
                    str_out += '# '
                _legend = self.legend[num]
                str_out += '@    s' + str(counter) + ' legend  "'
                str_out += _legend + '"\n'
                if not self.agr:
                    str_out += '# '
                str_out += '@    s' + str(counter)
                str_out += ' comment "' + self.filename + ' "\n'
                for i, ele in enumerate(self.indata):
                    str_out += self.xdata[i] + ' ' + str((float(ele[num]) * float(ele[num - 1]))) + '\n'
                out_file.write(str_out)

                counter += 1

# end special handling for 'm'


def main(args=sys.argv[1:]):
    parser = argparse.ArgumentParser()
    parser.add_argument("gd_file", help="location of gd file", type=str)
    parser.add_argument("dat_file", help="location of .dat to write", type=str, nargs='?')
    parser.add_argument("-v", "--verbosity", action='count', help="increase output verbosity", default=0)
    parser.add_argument('-V', '--version', action='version', version=pt.__version__)
    args = parser.parse_args(args)

    gd_data = ReadGd(args.gd_file, let=False, exp=True)
    gd_data.export(args.dat_file)

    return 0


if __name__ == '__main__':
    logging.basicConfig()
    sys.exit(main(sys.argv[1:]))

# TODO add also exporting LET data



#!/usr/bin/env python
#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Script for converting a DICOM file to TRiP98 / Voxelplan style files.
"""
import sys
import logging
import argparse

import pytrip as pt

logger = logging.getLogger(__name__)


def main(args=sys.argv[1:]):
    """ Main function for dicom2trip.py
    """
    # parse arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("dicom_folder", help="location of folder with DICOM files", type=str)
    parser.add_argument("ctx_basename", help="basename of output file in TRiP98 format", type=str)
    parser.add_argument("-v", "--verbosity", action='count', help="increase output verbosity", default=0)
    parser.add_argument('-V', '--version', action='version', version=pt.__version__)
    parsed_args = parser.parse_args(args)

    if parsed_args.verbosity == 1:
        logging.basicConfig(level=logging.INFO)
    elif parsed_args.verbosity > 1:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig()

    basename = parsed_args.ctx_basename

    # import DICOM
    dcm = pt.dicomhelper.read_dicom_folder(parsed_args.dicom_folder)

    if 'images' in dcm:
        c = pt.CtxCube()
        c.read_dicom(dcm)
        logger.info("Write CtxCube header... {}".format(basename + pt.CtxCube.header_file_extension))
        c.write_trip_header(basename + pt.CtxCube.header_file_extension)
        logger.info("Write CtxCube...        {}".format(basename + pt.CtxCube.data_file_extension))
        c.write_trip_data(basename + pt.CtxCube.data_file_extension)
    else:
        logger.warning("No CT data found in {}".format(parsed_args.dicom_folder))
        c = None

    if 'rtss' in dcm:
        logger.info("Write VdxCube...        {}".format(basename + ".vdx"))
        vdx_cube = pt.VdxCube(cube=c)
        vdx_cube.read_dicom(dcm)
        vdx_cube.write_trip(basename + ".vdx")
    else:
        logger.warning("No RTSTRUCT data found in {}".format(parsed_args.dicom_folder))

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
This module contains a parser method for parsing voxelplan files.
"""


def parse_to_var(data, var, stoptag=""):
    """ Parses a variable 'var' from 'data'.

    :returns: (out,i) tuple
    out - the rest of the line with 'var' and a " " removed. If nothing is followed, then True is returned.
    i - number of lines parsed.
    """
    out = {}
    i = 0
    if type(data) is str:
        data = data.split()
    for line in data:
        if line.find(stoptag) > -1:
            break
        items = line.split()
        if items[0] in var:  # if we have found 'var' in this line..
            if len(items) > 1:
                # remove "var" from that line, and returns the rest of that line.
                out[var[items[0]]] = line.replace(items[0] + " ", "")
            else:
                # in case nothing followed var. The presences of 'var' means smth is set to True.
                out[var[items[0]]] = True
        i += 1
    return out, i



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
This example demonstrates how to load a CT cube in Voxelplan format, and the associated contours.
Then a plan is prepared and optimized using TRiP98.
The resulting dose plan is then stored in <some_dir>.
"""

import pytrip as pt
import pytrip.tripexecuter as pte

# first define some paths and other important parameters
patient_name = "TST000000"
ctx_path = "/home/bassler/Projects/CTdata/TST000/TST000000.ctx"
vdx_path = "/home/bassler/Projects/CTdata/TST000/TST000000.vdx"
voi_name = "GTV"
working_directory = "/home/bassler/Projects/CTdata/TST000/"

ddd_dir = "/home/bassler/TRiP98/base/DATA/DDD/12C/RF3MM"
spc_dir = "/home/bassler/TRiP98/base/DATA/SPC/12C/RF3MM"
sis_path = "/home/bassler/TRiP98/base/DATA/SIS/12C.sis"
hlut_path = "/home/bassler/TRiP98/base/DATA/HLUT/19990218.hlut"
dedx_path = "/home/bassler/TRiP98/base/DATA/DEDX/20040607.dedx"

my_couch_angle = 90.0
my_gantry_angle = 10.0
my_target_voi = "GTV"  # the name must exist in the .vdx file
my_projectile = "C"  # carbon ion

# load CT cube
my_ctx = pt.CtxCube()
my_ctx.read(ctx_path)

# load VOIs
my_vdx = pt.VdxCube(my_ctx)  # my_vdx is the object which will hold all volumes of interest and the meta information
my_vdx.read(vdx_path)  # load the .vdx file
print(my_vdx.get_voi_names())  # show us all VOIs found in the .vdx file

# next pick a the proper VOI (from the VdxCube object) which we want to plan on
target_voi_temp = my_vdx.get_voi_by_name(my_target_voi)

# for technical reasons the voi must be cast into a new Voi object
# we are working on a cleaner Voi class implementation to avoid it
target_voi = pte.Voi("GTV_VOI", target_voi_temp)

# Next, setup a plan. We may initialize it with a new name.
# The name must be identical to the base name of the file, else we will have crash
my_plan = pte.TripPlan(name=patient_name)

# set working directory, output will go there
my_plan.set_working_dir(working_directory)
my_plan.set_ddd_folder(ddd_dir)
my_plan.set_spc_folder(spc_dir)
my_plan.set_dedx_file(dedx_path)
my_plan.set_hlut_file(hlut_path)
my_plan.set_sis_file(sis_path)

# To enable remote access to trip, uncomment and eddit the following:
# my_plan.set_remote_state(True)
# my_plan.set_server("titan.phys.au.dk")  # location of remote TRiP98 installation. Needs SSH access.
# my_plan.set_username("xxxxxxxxx")
# my_plan.set_password("xxxxxxxxx")  # to login using SSH-keys, leave this commented out.

# add target VOI to the plan
my_plan.add_voi(target_voi)
my_plan.get_vois()[0].target = True  # make TRiP98 aware of that this VOI is the target.

# Finally we need to add a field to the plan
# add default field, carbon ions
my_field = pte.Field("Field 1")
my_field.set_projectile(my_projectile)  # set the projectile
my_field.set_couch(my_couch_angle)
my_field.set_gantry(my_gantry_angle)

my_plan.add_field(my_field)

# the next line is needed to correctly set offset between VOI and CT
ct_images = pt.CTImages(my_ctx)

# run TRiP98 optimisation
my_trip = pte.TripExecuter(ct_images.get_modified_images(my_plan))
# TRiP98 will then run the plan and generate the requested dose plan.
# The dose plan is stored in the working directory, and must then be loaded by the user for further processing.
# for local execution, we assume TRiP98 binary is present in PATH env. variable
my_trip.execute(my_plan)



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
TODO: documentation here.
"""


def get_max(a, b):
    return [max(x, y) for x, y in zip(a, b)]


def get_min(a, b):
    return [min(x, y) for x, y in zip(a, b)]



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
This example shows how to use contours to select volume of interests inside a CTX cube. The VOI is then manipulated.
"""
import pytrip as pt

# first define some paths and other important parameters
ctx_path = "/home/bassler/Projects/CTdata/TST000/TST000000.ctx"
vdx_path = "/home/bassler/Projects/CTdata/TST000/TST000000.vdx"
my_target_voi = "GTV"

# load CT cube
my_ctx = pt.CtxCube()
my_ctx.read(ctx_path)

# load VOIs
my_vdx = pt.VdxCube(my_ctx)  # my_vdx is the object which will hold all volumes of interest and the meta information
my_vdx.read(vdx_path)  # load the .vdx file
print(my_vdx.get_voi_names())  # show us all VOIs found in the .vdx file

# Select the requested VOI from the VdxCube object
target_voi = my_vdx.get_voi_by_name(my_target_voi)

# get_voi_cube() returns a DosCube() object, where all voxels inside the VOI holds the value 1000, and 0 elsewhere.
voi_cube = target_voi.get_voi_cube()

# Based on the retrieved DosCube() we calculate a three dimensional mask object,
# which assigns True to all voxels inside the Voi, and False elsewhere.
mask = (voi_cube.cube == 1000)

# "The mask object and the CTX cube have same dimensions (they are infact inherited from the same top level class).
# Therefore we can apply the mask cube to the ctx cube and work with the values.
# For instance we can set all HUs to zero within the Voi:
my_ctx.cube[mask] = 0
# or add 100 to all HUs of the voxels inside the mask:
# my_ctx.cube[mask] += 100

# save masked CT to the file in current directory
masked_ctx = "masked.ctx"
my_ctx.write(masked_ctx)



#
#    Copyright (C) 2010-2016 PyTRiP98 Developers.
#
#    This file is part of PyTRiP98.
#
#    PyTRiP98 is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    PyTRiP98 is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PyTRiP98.  If not, see <http://www.gnu.org/licenses/>.
#
"""
TODO: documentation here.
"""
# do not check this file for PEP8 compatibility
# flake8 complains about "E402 module level import not at top of file"
# flake8: noqa

import logging
from pytrip.ctx import CtxCube
from pytrip.dos import DosCube
from pytrip.vdx import VdxCube, Voi
from pytrip.paths import DensityCube
from pytrip.raster import Rst
from pytrip.let import LETCube
from pytrip.ctimage import CTImages
from pytrip.dicomhelper import read_dicom_folder

# from https://docs.python.org/3/tutorial/modules.html
# if a package's __init__.py code defines a list named __all__,
# it is taken to be the list of module names that should be imported when from package import * is encountered.
__all__ = ['CtxCube', 'VdxCube', 'Voi', 'DosCube', 'DensityCube', 'LETCube', 'dicomhelper', 'res',
           'Rst', 'CTImages']

# if an application using pytrip doesn't configure any logging level, then an error will occur
# to prevent it, we add null logging handler, as suggested by Python documentation:
# as described here: https://docs.python.org/3/howto/logging.html#library-config
logging.getLogger(__name__).addHandler(logging.NullHandler())

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions
